#--------------------------------------------------------------------------
#                  Copyright (c) Donald Syme 1992                          
#                  All rights reserved                                     
#                                                                          
# Donald Syme, hereafter referred to as `the Author', retains the copyright
# and all other legal rights to the Software contained in this file,       
# hereafter referred to as `the Software'.                                 
#                                                                          
# The Software is made available free of charge on an `as is' basis. No    
# guarantee, either express or implied, of maintenance, reliability,       
# merchantability or suitability for any purpose is made by the Author.    
#                                                                          
# The user is granted the right to make personal or internal use of the    
# Software provided that both:                                             
# 1. The Software is not used for commercial gain.                         
# 2. The user shall not hold the Author liable for any consequences        
#    arising from use of the Software.                                     
#                                                                          
# The user is granted the right to further distribute the Software         
# provided that both:                                                      
# 1. The Software and this statement of rights are not modified.           
# 2. The Software does not form part or the whole of a system distributed  
#    for commercial gain.                                                  
#                                                                          
# The user is granted the right to modify the Software for personal or     
# internal use provided that all of the following conditions are observed: 
# 1. The user does not distribute the modified software.                   
# 2. The modified software is not used for commercial gain.                
# 3. The Author retains all rights to the modified software.               
#                                                                          
# Anyone seeking a licence to use this software for commercial purposes is 
# invited to contact the Author.                                           
#--------------------------------------------------------------------------




 #########################################################################
 #                                                                       #
 # Copyright (C) 1993 by General Electric company.  All rights reserved. #
 #                                                                       #
 # Permission to use, copy, modify, and distribute this                  #
 # software and its documentation for any purpose and without            #
 # fee is hereby granted, provided that the above copyright              #
 # notice appear in all copies and that both that copyright              #
 # notice and this permission notice appear in supporting                #
 # documentation, and that the name of General Electric not be used in   #
 # advertising or publicity pertaining to distribution of the            #
 # software without specific, written prior permission.                  #
 #                                                                       #
 # General Electric makes no representations about the suitability of    #
 # this software for any purpose.  It is provided ``as is''              #
 # without express or implied warranty.                                  #
 #                                                                       #
 # This work was supported in part by the DARPA Initiative in Concurrent #
 # Engineering (DICE) through DARPA Contracts MDA972-88-C-0047 and       #
 # MDA972-92-C-0027.                                                     #
 #                                                                       #
 # This work was supported in part by the Tri-Services Microwave and     #
 # Millimeter-Wave Advanced Computational Environment (MMACE) program    #
 # under Naval Research Laboratory contract N00014-92-C-2044.            #
 #                                                                       #
 #########################################################################



proc icon {w args} {

        set icon {}
        set fargs {}
        while {[llength $args] >= 2} {
                set option [lindex $args 0]
                set value [lindex $args 1]
                set args [lrange $args 2 end]
                case $option in {
                        -icon { set icon $value }
                        default {
                                lappend fargs $option $value
                        }
                }
        }
                if {$args != ""} {
                error "icon: extraneous argument $args"
        }

        if {$icon == ""} {
                error "icon: -icon argument missing"
        }

        set fileName [icon_find $icon]

        eval label $w [list -bitmap $fileName] $fargs

        return $w
}

# Procedure:    icon_find
#
# Synopsis:
#       Search the auto-load path for a bitmap
#
# Usage:
#c      icon_find iconName
#
# Parameters:
#c      iconName
#               Name of an icon whose bitmap is on the load path.
#
# Return value:
#       Bitmap (to use in a -bitmap option) of the icon.
#
# Description:
#       The `icon_find' command searches the load path for a named bitmap, 
#       and returns an appropriate bitmap name.  The file name is expected
#       to be
#
#c              iconName.xbm
#
#       where iconName is the name of the icon.
#
# Example:
#c      label .bomb -bitmap [icon_find bomb]
#               Creates a label with a picture of a bomb in it.

proc icon_find icon {
        global auto_path
        global icon_priv
        if {![info exists icon_priv(bitmap,$icon)]} {
                set fileName ""
                foreach dir $auto_path {
                        if [file readable $dir/$icon.xbm] {
                                set fileName $dir/$icon.xbm
                                break
                        }
                }
                set icon_priv(bitmap,$icon) $fileName
        } else {
                set fileName $icon_priv(bitmap,$icon)
        }
        if {$fileName == ""} {
                error "icon: icon `$icon' not found."
        }
         
        return @$fileName
}
